#!/usr/bin/perl
use strict;
use YAML::Tiny;
use IO::Socket;
use DBI;
use POSIX qw(strftime);
use DateTime;

my $stat_table = "level_stat";
my $level_table = "level_current";

my $config = YAML::Tiny->read( 'acs.cfg' );
my $mysql_cfg = $config->[0];
my $billing_cfg = $config->[1];
my $level_cfg = $config->[2];
my $level_time = $config->[3];

my $slave_cfg = $mysql_cfg->{slave};
my $slave_db = DBI->connect( $slave_cfg->{dsn}, $slave_cfg->{user}, $slave_cfg->{pswd} )
  or die "Cant connect to the Slave DB: $DBI::errstr\n"; 
 
my $stat_cfg = $mysql_cfg->{stat};
my $stat_db = DBI->connect( $stat_cfg->{dsn}, $stat_cfg->{user}, $stat_cfg->{pswd} )
  or die "Cant connect to the Stat DB: $DBI::errstr\n"; 

=pod
#print( "$level_cfg\n" );
for my $key ( keys %{ $level_cfg } )
{
  my $value = $level_cfg->{$key};
  print "$key => $value\n";
  
}
=cut

#print( "@($level_time)[0]\n" );

create_day_stat( DateTime->now()->subtract( "days" => 1 ) );
set_radius_levels();

#print( get_level( 390, "20,3" )."\n" );

$stat_db->disconnect();
$slave_db->disconnect();

sub set_radius_levels
{
   print( "Setting levels on RADIUS\n" );
   
   my $dt = DateTime->now( time_zone => `date +%z` );
   
   my $dw = $dt->day_of_week();
   my $hour = $dt->hour();
   
   print( "dw: $dw; hour: $hour \n" );
   
   my $set_rules = undef;   
   for my $rule ( @{ $level_time } )
   {
      my $dw_from = $rule->{dw_from};
      my $dw_to = $rule->{dw_to};
      my $hour_from = $rule->{hour_from};
      my $hour_to = $rule->{hour_to};
      
      if( $dw_from <= $dw && $dw <= $dw_to && 
	  $hour_from <= $hour && $hour <= $hour_to )
      {
	$set_rules = "true";
	print( "Sending levels\n" );
	last;
      }
   }
   
   my $sock = new IO::Socket::INET ( PeerAddr => $billing_cfg->{radius_host}, PeerPort => $billing_cfg->{radius_port}, Proto => 'tcp' ); 
   die "Could not create socket: $!\n" unless $sock; 
   
   print $sock "set_levels\n";
   
   if( $set_rules )
   {
       my $query = "SELECT cid, level FROM $level_table WHERE level>0";
       my $sth = $stat_db->prepare( $query );
       $sth->execute();
       
       while( my @row = $sth->fetchrow_array() )
       {
	  print $sock "$row[0]\t$row[1]\n";
       }
   }
   
   close( $sock );
}

sub create_day_stat
{
  my $dt = shift;

  my $dtYYYYmm = $dt->strftime( "%Y%m" );
  my $dtSQL = "'".$dt->strftime( "%Y-%m-%d" )."'";
  my $mid = $billing_cfg->{mid};
  my $sid = $billing_cfg->{sid};
  
  my $detail_table = "session_detail_${mid}_$dtYYYYmm";
  my $period_table = "calculate_period_$mid";
  my $tariff_table = "contract_tariff";
  
  my @rows = $stat_db->selectrow_array( "SELECT * FROM $stat_table WHERE dt=$dtSQL LIMIT 1" );
  if( $#rows <= 0 )
  {
      print( "Filling $stat_table on $dtSQL\n" );
      
      my $query = 
	 "INSERT INTO $stat_table (dt, cid, level) VALUES ($dtSQL, ?, ?)";
      my $sth_insert = $stat_db->prepare( $query );
      
      $query =
	  "SELECT detail.cid, ROUND( SUM(amount) / (1024*1024*1024), 2), tariff.tpid FROM $detail_table AS detail \
	  LEFT JOIN $tariff_table AS tariff \
	  ON detail.cid=tariff.cid AND tariff.date1<=$dtSQL AND (tariff.date2 IS NULL OR $dtSQL<=tariff.date2) \
	  WHERE detail.sid=$sid AND $dtSQL<=detail.dtime AND detail.dtime<DATE_ADD($dtSQL, INTERVAL 1 DAY) \
	  GROUP BY cid"; 
	   
      my $sth = $slave_db->prepare( $query );
      $sth->execute(); 
      
      print( "Select finished, inserting stat info\n" );
      
      my $cnt = 0;
      
      while( my @row = $sth->fetchrow_array() )
      {
	my $cid = $row[0];
	my $amount = $row[1];
	my $tpid = $row[2];
	
	my $level = get_level( $tpid, $amount );
	$sth_insert->execute( $cid, $level );
	
	if( ++$cnt % 1000 == 0 )
	{
	  print( "$cnt\n" );
	}
      }
      
      print( "Updating current level\n" );
      update_current_level();
  }
  else
  {
    print( "Stat data on $dtSQL already exist.\n" );
  }
}

sub get_level
{
  my $result = 0;

  my $tpid = shift;
  my $amount = shift;
  
  my $tariff_cfg = $level_cfg->{$tpid};
  if( $tariff_cfg )
  {
    #print( "$tpid : $amount\n" );
  
    for my $rule ( @{$tariff_cfg} )
    {
      my ($amount_from, $amount_to, $rule_level) = split( /\s*\,\s*/, $rule );
      
      #print( "$amount_from : $amount_to : $rule_level\n" );
      
      if( $amount_from <= $amount && 
	  ($amount < $amount_to || $amount_to == 0 ) )
      {
	$result = $rule_level;
	last;
      }
    }
  }
  else
  {
    #print( "Not found tariff cfg for tariff $tpid\n" );
  }
  
  return $result;
}

sub update_current_level
{
  $stat_db->do( "DELETE FROM $level_table" );  
  $stat_db->do( "INSERT INTO $level_table (cid, level) SELECT cid, AVG(level) FROM $stat_table GROUP BY cid" );
}